@@ -26,8 +26,6 @@
 #include "gnutls_num.h"
 #include "gnutls_gcry.h"
 
-#define DEFAULT_BITS 1024
-
 int gen_anon_server_kx( GNUTLS_STATE, opaque**);
 int gen_anon_client_kx( GNUTLS_STATE, opaque**);
 int proc_anon_server_kx( GNUTLS_STATE, opaque*, int);
@@ -84,26 +82,36 @@ int gen_anon_server_kx( GNUTLS_STATE state, opaque** data) {
 	}
 
 	g = gnutls_get_dh_params(&p, bits);
-	if (g==NULL) {
+	if (g==NULL || p==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 	state->gnutls_key->auth_info = gnutls_malloc(sizeof(ANON_SERVER_AUTH_INFO));
 	if (state->gnutls_key->auth_info==NULL) return GNUTLS_E_MEMORY_ERROR;
+
 	((ANON_SERVER_AUTH_INFO)state->gnutls_key->auth_info)->dh_bits = gcry_mpi_get_nbits(p);
 	state->gnutls_key->auth_info_size = sizeof(ANON_SERVER_AUTH_INFO_INT);
 
 	X = gnutls_calc_dh_secret(&x, g, p);
-	if (X==NULL) 
+	if (X==NULL || x==NULL) {
+		gnutls_assert();
+		_gnutls_mpi_release( &g);
+		_gnutls_mpi_release( &p);
+		_gnutls_mpi_release( &x);
+		_gnutls_mpi_release( &X);
 		return GNUTLS_E_MEMORY_ERROR;
-
+	}
+	
 	state->gnutls_key->dh_secret = x;
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_g, g);
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_p, p);
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_X, X);
 	(*data) = gnutls_malloc(n_g + n_p + n_X + 6);
 	if (*data==NULL) {
+		_gnutls_mpi_release( &X);
+		_gnutls_mpi_release( &g);
+		_gnutls_mpi_release( &p);
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 	data_p = &(*data)[0];
@@ -149,7 +157,7 @@ int ret;
 	_gnutls_mpi_release(&X);
 	
 	WRITEuint16( n_X, &(*data)[0]);
-	
+
 	/* calculate the key after calculating the message */
 	state->gnutls_key->KEY = gnutls_calc_dh_key(state->gnutls_key->client_Y, x, state->gnutls_key->client_p);
 	if (state->gnutls_key->KEY==NULL)
@@ -255,15 +263,6 @@ int proc_anon_client_kx( GNUTLS_STATE state, opaque* data, int data_size) {
 		bits = cred->dh_bits;
 	}
 
-#if 0 /* removed. I do not know why - maybe I didn't get the protocol,
-       * but openssl does not use that byte
-       */
-	if (data[0] != 1) {
-		gnutls_assert();
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
-	}
-#endif
-
 	n_Y = READuint16( &data[0]);
 
 	_n_Y = n_Y;
