@@ -14,3 +14,5 @@ typedef struct {
 GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_KEY key, char* username, int* err);
 void _gnutls_srp_clear_pwd_entry( GNUTLS_SRP_PWD_ENTRY * entry);
 GNUTLS_SRP_PWD_ENTRY* _gnutls_randomize_pwd_entry();
+int _gnutls_sbase64_encode(uint8 * data, int data_size, uint8 ** result);
+int _gnutls_sbase64_decode(uint8 * data, int data_size, uint8 ** result);
