@@ -414,7 +414,7 @@ int _gnutls_gen_x509_client_certificate(GNUTLS_STATE state, opaque ** data)
 	gnutls_private_key *apr_pkey;
 	int apr_cert_list_length;
 
-
+#warning GET_CLIENT_CERTIFICATE
 	/* find the appropriate certificate */
 	if ((ret=_gnutls_find_apr_cert( state, &apr_cert_list, &apr_cert_list_length, &apr_pkey))<0) {
 		gnutls_assert();
@@ -466,6 +466,7 @@ int _gnutls_gen_x509_server_certificate(GNUTLS_STATE state, opaque ** data)
 	gnutls_private_key *apr_pkey;
 	int apr_cert_list_length;
 
+#warning GET_SERVER_CERTIFICATE
 	if ((ret=_gnutls_find_apr_cert( state, &apr_cert_list, &apr_cert_list_length, &apr_pkey))<0) {
 		gnutls_assert();
 		return ret;
@@ -523,6 +524,7 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data, int
 	gnutls_datum tmp;
 	CertificateStatus verify;
 
+#warning PUT_SERVER_CERTIFICATE
 	cred = _gnutls_get_cred(state->gnutls_key, GNUTLS_X509PKI, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
@@ -615,6 +617,7 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data, int
 		j++;
 	}
 
+#warning GET_SERVER_CERTIFICATE_PARAMS
 	/* store the required parameters for the handshake
 	 */
 	if ((ret =
@@ -747,6 +750,7 @@ int _gnutls_gen_x509_client_cert_vrfy(GNUTLS_STATE state, opaque ** data)
 
 	*data = NULL;
 	
+#warning GENERATE_SIGNATURE_FROM_DATA
 	/* find the appropriate certificate */
 	if ((ret=_gnutls_find_apr_cert( state, &apr_cert_list, &apr_cert_list_length, &apr_pkey))<0) {
 		gnutls_assert();
@@ -789,6 +793,7 @@ gnutls_datum sig;
 	size = READuint16( pdata);
 	pdata += 2;
 
+#warning VERIFY_SIGNATURE_FROM_DATA
 	if ( size < data_size - 2) {
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
@@ -847,6 +852,7 @@ int _gnutls_gen_x509_server_cert_req(GNUTLS_STATE state, opaque ** data)
 	pdata += 2;
 	size += 2;
 
+#warning GET_TRUSTED_CAS_DN
 	for (i = 0; i < cred->ncas; i++) {
 		if ( (ret=_gnutls_find_dn( &dn, &cred->ca_list[i])) < 0) {
 			gnutls_free( (*data));
