@@ -49,10 +49,6 @@
 # include <gdbm.h>
 #endif
 
-#ifdef USE_GCRYPT
-# include <gcrypt.h>
-#endif
-
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
@@ -65,13 +61,16 @@
 # include <sys/socket.h>
 #endif
 
+#ifdef LIBMCRYPT24
+# define USE_MCRYPT
+#endif
+
 #if SIZEOF_UNSIGNED_LONG_INT == 8
 # define HAVE_UINT64
 /* only used native uint64 in 64 bit machines */
  typedef unsigned long int uint64;
- typedef signed long int sint64;
 #else
-typedef struct { unsigned char i[8]; } uint64;
+ typedef struct { unsigned char i[8]; } uint64;
 #endif
 
 
