@@ -25,7 +25,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.2.9"
+#define LIBGNUTLS_VERSION "0.2.10"
 
 #include <sys/types.h>
 
@@ -61,7 +61,7 @@ typedef enum GNUTLS_Version { GNUTLS_SSL3=1, GNUTLS_TLS1 } GNUTLS_Version;
  */
 #define SOCKET int
 
-#define LIST ...
+#define GNUTLS_LIST ...
 
 struct GNUTLS_STATE_INT;
 typedef struct GNUTLS_STATE_INT* GNUTLS_STATE;
@@ -119,11 +119,11 @@ ssize_t gnutls_write(SOCKET cd, GNUTLS_STATE state, void *data, size_t sizeofdat
 ssize_t gnutls_read(SOCKET cd, GNUTLS_STATE state, void *data, size_t sizeofdata);
 
 /* functions to set priority of cipher suites */
-int gnutls_set_cipher_priority( GNUTLS_STATE state, LIST);
-int gnutls_set_mac_priority( GNUTLS_STATE state, LIST);
-int gnutls_set_compression_priority( GNUTLS_STATE state, LIST);
-int gnutls_set_kx_priority( GNUTLS_STATE state, LIST);
-int gnutls_set_protocol_priority( GNUTLS_STATE state, LIST);
+int gnutls_set_cipher_priority( GNUTLS_STATE state, GNUTLS_LIST);
+int gnutls_set_mac_priority( GNUTLS_STATE state, GNUTLS_LIST);
+int gnutls_set_compression_priority( GNUTLS_STATE state, GNUTLS_LIST);
+int gnutls_set_kx_priority( GNUTLS_STATE state, GNUTLS_LIST);
+int gnutls_set_protocol_priority( GNUTLS_STATE state, GNUTLS_LIST);
 
 /* set our version - 0 for TLS 1.0 and 1 for SSL3 */
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state);
@@ -136,10 +136,19 @@ int gnutls_get_current_session( GNUTLS_STATE state, void* session, int *session_
 /* returns the session ID */
 int gnutls_get_current_session_id( GNUTLS_STATE state, void* session, int *session_size);
 
+typedef int (*DB_STORE_FUNC)(void*, gnutls_datum key, gnutls_datum data);
+typedef int (*DB_REMOVE_FUNC)(void*, gnutls_datum key);
+typedef gnutls_datum (*DB_RETR_FUNC)(void*, gnutls_datum key);
+
 int gnutls_set_lowat( GNUTLS_STATE state, int num);
 int gnutls_set_cache_expiration( GNUTLS_STATE state, int seconds);
 int gnutls_set_db_name( GNUTLS_STATE state, char* filename);	
 int gnutls_clean_db( GNUTLS_STATE state);
+void gnutls_set_db_retrieve_func( GNUTLS_STATE, DB_RETR_FUNC);
+void gnutls_set_db_remove_func( GNUTLS_STATE, DB_REMOVE_FUNC);
+void gnutls_set_db_store_func( GNUTLS_STATE, DB_STORE_FUNC);
+void gnutls_set_db_ptr( GNUTLS_STATE, void* db_ptr);
+int gnutls_check_db_entry( GNUTLS_STATE state, gnutls_datum session_entry);
 
 void gnutls_set_max_handshake_data_buffer_size( GNUTLS_STATE state, int max);
 
@@ -222,10 +231,12 @@ int gnutls_dh_generate_params( gnutls_datum* prime, gnutls_datum* generator, int
 
 typedef ssize_t (*PULL_FUNC)(SOCKET, void*, size_t);
 typedef ssize_t (*PUSH_FUNC)(SOCKET, const void*, size_t);
+
 typedef void (*LOG_FUNC)( const char*);
 
-void gnutls_global_set_push_func( PUSH_FUNC push_func);
-void gnutls_global_set_pull_func( PULL_FUNC pull_func);
+void gnutls_set_push_func( GNUTLS_STATE, PUSH_FUNC push_func);
+void gnutls_set_pull_func( GNUTLS_STATE, PULL_FUNC pull_func);
+
 
 
 #ifndef GNUTLS_UI_H
