@@ -21,6 +21,7 @@
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
 #include <gnutls_num.h>
+#include <gnutls_record.h>
 
 /* This is the only file that uses the berkeley sockets API.
  */
@@ -37,13 +38,10 @@
 # include <io_debug.h>
 #endif
 
-extern ssize_t (*_gnutls_pull_func)( SOCKET, void*, size_t);
-extern ssize_t (*_gnutls_push_func)( SOCKET,const void*, size_t);
-
 /* Buffers received packets of type APPLICATION DATA and
  * HANDSHAKE DATA.
  */
-int gnutls_insertDataBuffer(ContentType type, GNUTLS_STATE state, char *data, int length)
+int gnutls_insert_to_data_buffer(ContentType type, GNUTLS_STATE state, char *data, int length)
 {
 	int old_buffer;
 
@@ -77,7 +75,7 @@ int gnutls_insertDataBuffer(ContentType type, GNUTLS_STATE state, char *data, in
 
 }
 
-int gnutls_getDataBufferSize(ContentType type, GNUTLS_STATE state)
+int gnutls_get_data_buffer_size(ContentType type, GNUTLS_STATE state)
 {
 	if (type == GNUTLS_APPLICATION_DATA)
 		return state->gnutls_internals.buffer.size;
@@ -98,10 +96,10 @@ int gnutls_getDataBufferSize(ContentType type, GNUTLS_STATE state)
   * to work).
   **/
 int gnutls_check_pending(GNUTLS_STATE state) {
-	return gnutls_getDataBufferSize(GNUTLS_APPLICATION_DATA, state);
+	return gnutls_get_data_buffer_size(GNUTLS_APPLICATION_DATA, state);
 }
 
-int gnutls_getDataFromBuffer(ContentType type, GNUTLS_STATE state, char *data, int length)
+int gnutls_get_data_buffer(ContentType type, GNUTLS_STATE state, char *data, int length)
 {
 	if (type == GNUTLS_APPLICATION_DATA) {
 	
@@ -151,7 +149,7 @@ int gnutls_getDataFromBuffer(ContentType type, GNUTLS_STATE state, char *data, i
  *
  * Flags are only used if the default recv() function is being used.
  */
-static ssize_t _gnutls_read(SOCKET fd, void *iptr, size_t sizeOfPtr, int flags)
+static ssize_t _gnutls_read( SOCKET fd, GNUTLS_STATE state, void *iptr, size_t sizeOfPtr, int flags)
 {
 	size_t left;
 	ssize_t i=0;
@@ -164,10 +162,10 @@ static ssize_t _gnutls_read(SOCKET fd, void *iptr, size_t sizeOfPtr, int flags)
 	left = sizeOfPtr;
 	while (left > 0) {
 		
-		if (_gnutls_pull_func==NULL)
-			i = recv(fd, &ptr[i], left, flags);
+		if (state->gnutls_internals._gnutls_pull_func==NULL)
+			i = recv(fd, &ptr[sizeOfPtr-left], left, flags);
 		else
-			i = _gnutls_pull_func(fd, &ptr[i], left);
+			i = state->gnutls_internals._gnutls_pull_func(fd, &ptr[sizeOfPtr-left], left);
 				
 		if (i < 0) {
 #ifdef READ_DEBUG
@@ -191,7 +189,6 @@ static ssize_t _gnutls_read(SOCKET fd, void *iptr, size_t sizeOfPtr, int flags)
 #ifdef READ_DEBUG
 			_gnutls_log( "READ: Got %d bytes from %d\n", i, fd);
 #endif
-
 			if (i == 0)
 				break;	/* EOF */
 		}
@@ -248,7 +245,7 @@ int ret, sum;
         /* this was already read by using MSG_PEEK - so it shouldn't fail */
 	sum = 0;
         do { /* we need this to finish now */
-        	ret = _gnutls_read( cd, peek, RCVLOWAT-sum, 0);
+        	ret = _gnutls_read( cd, state, peek, RCVLOWAT-sum, 0);
         	if (ret > 0) sum+=ret;
        	} while( ret==GNUTLS_E_INTERRUPTED || ret==GNUTLS_E_AGAIN || sum < RCVLOWAT);
 
@@ -332,7 +329,7 @@ ssize_t _gnutls_read_buffered( int fd, GNUTLS_STATE state, opaque **iptr, size_t
 	/* READ DATA - but leave RCVLOWAT bytes in the kernel buffer.
 	 */
 	if ( recvdata - recvlowat > 0) {
-		ret = _gnutls_read( fd, &buf[buf_pos], recvdata - recvlowat, 0);
+		ret = _gnutls_read( fd, state, &buf[buf_pos], recvdata - recvlowat, 0);
 
 		/* return immediately if we got an interrupt or eagain
 		 * error.
@@ -359,7 +356,7 @@ ssize_t _gnutls_read_buffered( int fd, GNUTLS_STATE state, opaque **iptr, size_t
 	 * select think, that the socket is ready for reading
 	 */
 	if (ret == (recvdata - recvlowat) && recvlowat > 0) {
-		ret2 = _gnutls_read( fd, &buf[buf_pos], recvlowat, MSG_PEEK);
+		ret2 = _gnutls_read( fd, state, &buf[buf_pos], recvlowat, MSG_PEEK);
 
 		if (ret2 < 0 && gnutls_is_fatal_error(ret2)==0) {
 			return ret2;
@@ -408,6 +405,8 @@ ssize_t _gnutls_read_buffered( int fd, GNUTLS_STATE state, opaque **iptr, size_t
 	}
 }
 
+
+
 /* This function is like write. But it does not return -1 on error.
  * It does return gnutls_errno instead.
  *
@@ -419,7 +418,7 @@ ssize_t _gnutls_read_buffered( int fd, GNUTLS_STATE state, opaque **iptr, size_t
  * to decrypt and verify the integrity. 
  *
  */
-ssize_t _gnutls_write_buffered(int fd, GNUTLS_STATE state, const void *iptr, size_t n)
+ssize_t _gnutls_write_buffered(SOCKET fd, GNUTLS_STATE state, const void *iptr, size_t n)
 {
 	size_t left;
 #ifdef WRITE_DEBUG
@@ -442,23 +441,26 @@ ssize_t _gnutls_write_buffered(int fd, GNUTLS_STATE state, const void *iptr, siz
 	/* If data in the buffer exist
 	 */
 	if (iptr == NULL) {
-		if ( state->gnutls_internals.send_buffer.size == 0) {
-			gnutls_assert();
-			return GNUTLS_E_INVALID_PARAMETERS;
-		} else {
-			ptr = state->gnutls_internals.send_buffer.data;
-			n = state->gnutls_internals.send_buffer.size;
-		}
+		/* checking is handled above */
+		ptr = state->gnutls_internals.send_buffer.data;
+		n = state->gnutls_internals.send_buffer.size;
+#ifdef WRITE_DEBUG
+		_gnutls_log( "WRITE: Restoring old write. (%d data to send)\n", n);
+#endif
 	}
 
+#ifdef WRITE_DEBUG
+	_gnutls_log( "WRITE: Will write %d bytes to %d.\n", n, fd);
+#endif
+
 	i = 0;
 	left = n;
 	while (left > 0) {
 		
-		if (_gnutls_push_func==NULL) 
-			i = send(fd, &ptr[i], left, 0);
+		if (state->gnutls_internals._gnutls_push_func==NULL) 
+			i = send(fd, &ptr[n-left], left, 0);
 		else
-			i = _gnutls_push_func(fd, &ptr[i], left);
+			i = state->gnutls_internals._gnutls_push_func(fd, &ptr[n-left], left);
 
 		if (i == -1) {
 			if (errno == EAGAIN || errno == EINTR) {
@@ -470,20 +472,37 @@ ssize_t _gnutls_write_buffered(int fd, GNUTLS_STATE state, const void *iptr, siz
 					return GNUTLS_E_MEMORY_ERROR;
 				}
 				state->gnutls_internals.send_buffer.size = left;
-				memcpy( state->gnutls_internals.send_buffer.data, &ptr[n-left], left);
+				/* use memmove since they may overlap 
+				 */
+				memmove( state->gnutls_internals.send_buffer.data, &ptr[n-left], left);
 #ifdef WRITE_DEBUG
-				_gnutls_log( "WRITE: Interrupted. wrote %d bytes to %d. Left %d\n", n-left, fd, left);
+				_gnutls_log( "WRITE: Interrupted.\n");
 #endif
 				gnutls_assert();
-				if (errno==EAGAIN) return GNUTLS_E_AGAIN;
-				else return GNUTLS_E_INTERRUPTED;
+				if (errno==EAGAIN) retval = GNUTLS_E_AGAIN;
+				else retval = GNUTLS_E_INTERRUPTED;
 
+				return retval;
 			} else {
 				gnutls_assert();
 				return GNUTLS_E_PUSH_ERROR;
 			}
 		}
 		left -= i;
+
+#ifdef WRITE_DEBUG
+		_gnutls_log( "WRITE: wrote %d bytes to %d. Left %d bytes\n", i, fd, left);
+		for (x=0;x<((n-left)/16)+1;x++) {
+			_gnutls_log( "%.4x - ",x);
+			for (j=0;j<16;j++) {
+				if (sum<n-left) {
+					_gnutls_log( "%.2x ", ((unsigned char*)ptr)[sum++]);
+				}
+			}
+			_gnutls_log( "\n");
+		}
+#endif
+
 	}
 
 	retval = n + state->gnutls_internals.send_buffer_prev_size;
@@ -491,38 +510,44 @@ ssize_t _gnutls_write_buffered(int fd, GNUTLS_STATE state, const void *iptr, siz
 	state->gnutls_internals.send_buffer.size = 0;
 	state->gnutls_internals.send_buffer_prev_size = 0;
 
-#ifdef WRITE_DEBUG
-	_gnutls_log( "WRITE: wrote %d bytes to %d\n", n, fd);
-	for (x=0;x<(n/16)+1;x++) {
-		_gnutls_log( "%.4x - ",x);
-		for (j=0;j<16;j++) {
-			if (sum<n) {
-				_gnutls_log( "%.2x ", ((unsigned char*)ptr)[sum++]);
-			}
-		}
-		_gnutls_log( "\n");
-	
-	}
-#endif
 	return retval;
 
 }
 
+/* This function writes the data that are left in the
+ * TLS write buffer (ie. because the previous write was
+ * interrupted.
+ */
+ssize_t _gnutls_flush(SOCKET fd, GNUTLS_STATE state)
+{
+    ssize_t ret;
+
+    if (state->gnutls_internals.send_buffer.size == 0)
+        return 0; /* done */
+
+    ret = _gnutls_write_buffered(fd, state, NULL, 0);
+#ifdef WRITE_DEBUG 
+    _gnutls_log("WRITE FLUSH: %d\n", ret);
+#endif
+    return ret;
+}
+
+
 /* This is a send function for the gnutls handshake 
  * protocol. Just makes sure that all data have been sent.
  */
-ssize_t _gnutls_handshake_send_int(int fd, GNUTLS_STATE state, ContentType type, HandshakeType htype, void *iptr, size_t n)
+ssize_t _gnutls_handshake_send_int( SOCKET fd, GNUTLS_STATE state, ContentType type, HandshakeType htype, void *iptr, size_t n)
 {
 	size_t left;
 	ssize_t i = 0;
 	char *ptr = iptr;
 
 	if (iptr==NULL && n == 0) {
-		uint8 sdata = 0;
 		/* resuming interrupted write. Put some random data into
 		 * the data field so send_int() will proceed normally.
 		 */
-		return gnutls_send_int( fd, state, type, htype, &sdata, 1);
+		return _gnutls_flush( fd, state);
+//		return gnutls_send_int( fd, state, type, htype, &sdata, 1);
 	}
 
 	left = n;
@@ -578,7 +603,7 @@ ssize_t _gnutls_handshake_recv_int(int fd, GNUTLS_STATE state, ContentType type,
 /* Buffer for handshake packets. Keeps the packets in order
  * for finished messages to use them.
  */
-int gnutls_insertHashDataBuffer( GNUTLS_STATE state, char *data, int length)
+int gnutls_insert_to_handshake_buffer( GNUTLS_STATE state, char *data, int length)
 {
 	int old_buffer;
 
@@ -607,13 +632,13 @@ int gnutls_insertHashDataBuffer( GNUTLS_STATE state, char *data, int length)
 	return 0;
 }
 
-int gnutls_getHashDataBufferSize( GNUTLS_STATE state)
+int gnutls_get_handshake_buffer_size( GNUTLS_STATE state)
 {
 
 	return state->gnutls_internals.hash_buffer.size;
 }
 
-int gnutls_getHashDataFromBuffer( GNUTLS_STATE state, char *data, int length)
+int gnutls_get_handshake_buffer( GNUTLS_STATE state, char *data, int length)
 {
 	if (length > state->gnutls_internals.hash_buffer.size) {
 		length = state->gnutls_internals.hash_buffer.size;
@@ -641,8 +666,9 @@ int gnutls_getHashDataFromBuffer( GNUTLS_STATE state, char *data, int length)
 }
 
 /* this function does not touch the buffer
+ * and returns data from it (peek mode!)
  */
-int gnutls_readHashDataFromBuffer( GNUTLS_STATE state, char *data, int length)
+int gnutls_read_handshake_buffer( GNUTLS_STATE state, char *data, int length)
 {
 	if (length > state->gnutls_internals.hash_buffer.size) {
 		length = state->gnutls_internals.hash_buffer.size;
@@ -656,7 +682,7 @@ int gnutls_readHashDataFromBuffer( GNUTLS_STATE state, char *data, int length)
 
 
 
-int gnutls_clearHashDataBuffer( GNUTLS_STATE state)
+int gnutls_clear_handshake_buffer( GNUTLS_STATE state)
 {
 
 #ifdef BUFFERS_DEBUG
