@@ -18,9 +18,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-int gnutls_insertDataBuffer(ContentType type, GNUTLS_STATE state, char *data, int length);
-int gnutls_getDataBufferSize(ContentType type, GNUTLS_STATE state);
-int gnutls_getDataFromBuffer(ContentType type, GNUTLS_STATE state, char *data, int length);
+int gnutls_insert_to_data_buffer(ContentType type, GNUTLS_STATE state, char *data, int length);
+int gnutls_get_data_buffer_size(ContentType type, GNUTLS_STATE state);
+int gnutls_get_data_buffer(ContentType type, GNUTLS_STATE state, char *data, int length);
 ssize_t _gnutls_read_buffered(int fd, GNUTLS_STATE, opaque **iptr, size_t n, ContentType);
 void _gnutls_read_clear_buffer( GNUTLS_STATE);
 int _gnutls_clear_peeked_data( SOCKET cd, GNUTLS_STATE state);
@@ -28,11 +28,12 @@ int _gnutls_clear_peeked_data( SOCKET cd, GNUTLS_STATE state);
 ssize_t _gnutls_write_buffered(int fd, GNUTLS_STATE, const void *iptr, size_t n );
 
 /* used in SSL3 */
-int gnutls_getHashDataFromBuffer( GNUTLS_STATE state, char *data, int length);
-int gnutls_getHashDataBufferSize( GNUTLS_STATE state);
-int gnutls_readHashDataFromBuffer( GNUTLS_STATE state, char *data, int length);
-int gnutls_insertHashDataBuffer( GNUTLS_STATE state, char *data, int length);
-int gnutls_clearHashDataBuffer( GNUTLS_STATE state);
+int gnutls_get_handshake_buffer( GNUTLS_STATE state, char *data, int length);
+int gnutls_get_handshake_buffer_size( GNUTLS_STATE state);
+int gnutls_read_handshake_buffer( GNUTLS_STATE state, char *data, int length);
+int gnutls_insert_to_handshake_buffer( GNUTLS_STATE state, char *data, int length);
+int gnutls_clear_handshake_buffer( GNUTLS_STATE state);
 
-ssize_t _gnutls_handshake_recv_int(int fd, GNUTLS_STATE, ContentType, HandshakeType, void *, size_t);
-ssize_t _gnutls_handshake_send_int(int fd, GNUTLS_STATE, ContentType, HandshakeType, void *, size_t);
+ssize_t _gnutls_handshake_recv_int(SOCKET fd, GNUTLS_STATE, ContentType, HandshakeType, void *, size_t);
+ssize_t _gnutls_handshake_send_int(SOCKET fd, GNUTLS_STATE, ContentType, HandshakeType, void *, size_t);
+ssize_t _gnutls_flush(SOCKET fd, GNUTLS_STATE state);
