@@ -32,6 +32,7 @@
 #include <x509_verify.h>
 #include <x509_extensions.h>
 #include <gnutls_algorithms.h>
+#include <gnutls_dh.h>
 
 /* KX mappings to PK algorithms */
 typedef struct {
@@ -46,7 +47,7 @@ typedef struct {
  */
 static const gnutls_pk_map pk_mappings[] = {
 	{GNUTLS_KX_RSA, GNUTLS_PK_RSA},
-/*	{ GNUTLS_KX_DHE_RSA,     GNUTLS_PK_RSA }, */
+	{ GNUTLS_KX_DHE_RSA,     GNUTLS_PK_RSA }, 
 	{0}
 };
 
@@ -343,6 +344,9 @@ int gnutls_allocate_x509_sc(X509PKI_CREDENTIALS * res, int ncerts)
 	if (*res == NULL)
 		return GNUTLS_E_MEMORY_ERROR;
 
+
+	(*res)->dh_bits = DEFAULT_BITS;
+	
 	(*res)->ncerts = 0;	/* this is right - set_key() increments it */
 
 	if (ncerts > 0) {
@@ -428,6 +432,21 @@ int gnutls_set_x509_trust(X509PKI_CREDENTIALS res, char *CAFILE,
 	return 0;
 }
 
+/**
+  * gnutls_set_x509_dh_bits - Used to set the bits for a DHE_* ciphersuite
+  * @res: is an &X509PKI_CREDENTIALS structure.
+  * @bits: is the number of bits
+  *
+  * This function sets the number of bits, for use in a Diffie Hellman key exchange.
+  * This will only occur in a DHE ciphersuite.
+  *
+  **/
+int gnutls_set_x509_dh_bits(X509PKI_CREDENTIALS res, int bits)
+{
+	res->dh_bits = bits;
+	return 0;
+}
+
 
 static int _read_rsa_params(opaque * der, int dersize, MPI ** params)
 {
@@ -753,6 +772,13 @@ static int _gnutls_get_version(node_asn * c2, char *root)
 	return (int) gversion[0] + 1;
 }
 
+#ifdef DEBUG
+# warning FIX THIS FOR DSS
+#endif
+
+/* This function will convert a der certificate, to a format
+ * (structure) that gnutls can understand and use.
+ */
 int _gnutls_cert2gnutlsCert(gnutls_cert * gCert, gnutls_datum derCert)
 {
 	int result;
@@ -900,9 +926,9 @@ int _gnutls_cert2gnutlsCert(gnutls_cert * gCert, gnutls_datum derCert)
 }
 
 /* Returns 0 if it's ok to use the KXAlgorithm with this cert
- * (using KeyUsage field). 1 otherwise.
+ * (using KeyUsage field). 
  */
-static int _gnutls_check_x509_key_usage(gnutls_cert * cert,
+int _gnutls_check_x509_key_usage(gnutls_cert * cert,
 					KXAlgorithm alg)
 {
 	if (_gnutls_map_kx_get_cred(alg) == GNUTLS_X509PKI) {
@@ -912,7 +938,7 @@ static int _gnutls_check_x509_key_usage(gnutls_cert * cert,
 				if (!
 				    (cert->
 				     keyUsage & X509KEY_KEY_ENCIPHERMENT))
-					return 1;
+					return GNUTLS_E_X509_KEY_USAGE_VIOLATION;
 				else
 					return 0;
 			}
@@ -922,13 +948,14 @@ static int _gnutls_check_x509_key_usage(gnutls_cert * cert,
 				if (!
 				    (cert->
 				     keyUsage & X509KEY_DIGITAL_SIGNATURE))
-					return 1;
+					return GNUTLS_E_X509_KEY_USAGE_VIOLATION;
 				else
 					return 0;
 			}
 			return 0;
 		default:
-			return 1;
+			gnutls_assert();
+			return GNUTLS_E_X509_KEY_USAGE_VIOLATION;
 		}
 	}
 	return 0;
@@ -974,7 +1001,7 @@ int _gnutls_cert_supported_kx(gnutls_cert * cert, KXAlgorithm ** alg,
  * common_name (or subjectAltDNSName) field similar to name
  */
 gnutls_cert *_gnutls_find_cert(gnutls_cert ** cert_list,
-			       int cert_list_length, char *name)
+			       int cert_list_length, const char *name)
 {
 	gnutls_cert *cert = NULL;
 	int i;
@@ -999,7 +1026,7 @@ gnutls_cert *_gnutls_find_cert(gnutls_cert ** cert_list,
  * common_name (or subjectAltDNSName) field similar to name
  */
 int _gnutls_find_cert_list_index(gnutls_cert ** cert_list,
-				 int cert_list_length, char *name)
+				 int cert_list_length, const char *name)
 {
 	int index = 0;
 	int i;
@@ -1042,7 +1069,7 @@ int gnutls_x509pki_set_cert_request(GNUTLS_STATE state,
 /**
   * gnutls_set_x509_cert_callback - Used to set a callback while selecting the proper (client) certificate
   * @cred: is an &X509PKI_CLIENT_CREDENTIALS structure.
-  * @x509_cert_callback: is the callback function
+  * @func: is the callback function
   *
   * The callback's function form is:
   * int (*callback)(gnutls_DN *client_cert, gnutls_DN *issuer_cert, int ncerts, gnutls_DN* req_ca_cert, int nreqs);
@@ -1079,13 +1106,8 @@ int gnutls_x509pki_set_cert_request(GNUTLS_STATE state,
   *
   * This function returns 0 on success.
   **/
-int gnutls_set_x509_cert_callback(X509PKI_CREDENTIALS cred,
-				  int (*x509_cert_callback) (gnutls_DN *,
-							     gnutls_DN *,
-							     int,
-							     gnutls_DN *,
-							     int))
+int gnutls_set_x509_cert_callback(X509PKI_CREDENTIALS cred, x509_cert_callback_func* func)
 {
-	cred->client_cert_callback = x509_cert_callback;
+	cred->client_cert_callback = func;
 	return 0;
 }
