@@ -33,10 +33,12 @@
 #include "gnutls_record.h"
 #include "gnutls_constate.h"
 
-/* returns ciphertext which contains RECORD_HEADER_SIZE unused bytes
+/* returns ciphertext which contains the headers too. This also
+ * calculates the size in the header field.
  */
-int _gnutls_encrypt(GNUTLS_STATE state, const char *data, size_t data_size,
-		    uint8 ** ciphertext, ContentType type)
+int _gnutls_encrypt(GNUTLS_STATE state, const char* headers, int headers_size,
+		const char *data, size_t data_size,
+		uint8 ** ciphertext, ContentType type)
 {
 	gnutls_datum plain = { (char*)data, data_size };
 	gnutls_datum comp, ciph;
@@ -51,7 +53,7 @@ int _gnutls_encrypt(GNUTLS_STATE state, const char *data, size_t data_size,
 		return err;
 	}
 
-	err = _gnutls_compressed2TLSCiphertext(state, &ciph, comp, type);
+	err = _gnutls_compressed2TLSCiphertext(state, &ciph, comp, type, headers_size);
 	if (err < 0) {
 		gnutls_assert();
 		return err;
@@ -59,6 +61,10 @@ int _gnutls_encrypt(GNUTLS_STATE state, const char *data, size_t data_size,
 
 	gnutls_free_datum(&comp);
 
+	/* copy the headers */
+	memcpy( ciph.data, headers, headers_size);
+	WRITEuint16( ciph.size - headers_size, &ciph.data[3]);
+	
 	*ciphertext = ciph.data;
 
 	return ciph.size;
@@ -109,7 +115,7 @@ int _gnutls_decrypt(GNUTLS_STATE state, char *ciphertext,
 int _gnutls_compressed2TLSCiphertext(GNUTLS_STATE state,
 					gnutls_datum*
 					cipher,
-					gnutls_datum compressed, ContentType _type)
+					gnutls_datum compressed, ContentType _type, int headers_size)
 {
 	uint8 MAC[MAX_HASH_SIZE];
 	uint16 c_length;
@@ -117,7 +123,7 @@ int _gnutls_compressed2TLSCiphertext(GNUTLS_STATE state,
 	uint8 pad;
 	uint8 rand;
 	uint64 seq_num;
-	int length;
+	int length,ret;
 	GNUTLS_MAC_HANDLE td;
 	uint8 type = _type;
 	uint8 major, minor;
@@ -177,7 +183,7 @@ int _gnutls_compressed2TLSCiphertext(GNUTLS_STATE state,
 		length =
 		    compressed.size + hash_size;
 
-		data = gnutls_malloc(length+RECORD_HEADER_SIZE);
+		data = gnutls_malloc(length+headers_size);
 		if (data==NULL) {
 			gnutls_assert();
 			return GNUTLS_E_MEMORY_ERROR;
@@ -208,12 +214,14 @@ int _gnutls_compressed2TLSCiphertext(GNUTLS_STATE state,
 		pad = (uint8) (blocksize - (length % blocksize)) + rand;
 
 		length += pad;
-		data = gnutls_malloc(length+RECORD_HEADER_SIZE);
+
+		data = gnutls_malloc(length+headers_size);
 		if (data==NULL) {
 			gnutls_assert();
 			return GNUTLS_E_MEMORY_ERROR;
 		}
-		memset(&data[RECORD_HEADER_SIZE + length - pad], pad - 1, pad);
+		
+		memset(&data[headers_size + length - pad], pad - 1, pad);
 
 		break;
 	default:
@@ -221,16 +229,20 @@ int _gnutls_compressed2TLSCiphertext(GNUTLS_STATE state,
 		return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 	}
 
-	memcpy(&data[RECORD_HEADER_SIZE], compressed.data, compressed.size);
-	memcpy(&data[compressed.size+RECORD_HEADER_SIZE], MAC, hash_size);
+	memcpy(&data[headers_size], compressed.data, compressed.size);
+	memcpy(&data[compressed.size+headers_size], MAC, hash_size);
+
+	if ( (ret = gnutls_cipher_encrypt(state->connection_state.
+			      write_cipher_state, &data[headers_size], 
+			      length)) < 0) {
+		gnutls_free( data);
+		return ret;
+	}
 
-	gnutls_cipher_encrypt(state->connection_state.
-			      write_cipher_state, &data[RECORD_HEADER_SIZE], 
-			      length);
 	cipher->data = data;
-	cipher->size = length + RECORD_HEADER_SIZE;
-
+	cipher->size = length + headers_size;
 	return 0;
+
 }
 
 int _gnutls_ciphertext2TLSCompressed(GNUTLS_STATE state,
@@ -245,7 +257,7 @@ int _gnutls_ciphertext2TLSCompressed(GNUTLS_STATE state,
 	uint64 seq_num;
 	uint16 length;
 	GNUTLS_MAC_HANDLE td;
-	int blocksize;
+	int blocksize, ret;
 	uint8 major, minor;
 	int hash_size = _gnutls_mac_get_digest_size(state->security_parameters.read_mac_algorithm);
 
@@ -281,22 +293,16 @@ int _gnutls_ciphertext2TLSCompressed(GNUTLS_STATE state,
 
 	switch (_gnutls_cipher_is_block(state->security_parameters.read_bulk_cipher_algorithm)) {
 	case CIPHER_STREAM:
-		length =
-		    ciphertext.size - hash_size;
-		data = gnutls_malloc(length);
-		if (data==NULL) {
+		if ( (ret = gnutls_cipher_decrypt(state->connection_state.
+			      read_cipher_state, ciphertext.data,
+			      ciphertext.size)) < 0) {
 			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
+			return ret;
 		}
 
-		gnutls_cipher_decrypt(state->connection_state.
-				      read_cipher_state, ciphertext.data,
-				      ciphertext.size);
+		length =
+		    ciphertext.size - hash_size;
 
-		memcpy(data, ciphertext.data, length);
-
-		compress->data = data;
-		compress->size = length;
 		break;
 	case CIPHER_BLOCK:
 		if ((ciphertext.size < blocksize)
@@ -304,11 +310,16 @@ int _gnutls_ciphertext2TLSCompressed(GNUTLS_STATE state,
 			gnutls_assert();
 			return GNUTLS_E_DECRYPTION_FAILED;
 		}
-		gnutls_cipher_decrypt(state->connection_state.
-				      read_cipher_state, ciphertext.data,
-				      ciphertext.size);
+
+		if ( (ret = gnutls_cipher_decrypt(state->connection_state.
+			      read_cipher_state, ciphertext.data,
+			      ciphertext.size)) < 0) {
+			gnutls_assert();
+			return ret;
+		}
 
 		pad = ciphertext.data[ciphertext.size - 1] + 1;	/* pad */
+
 		length =
 		    ciphertext.size - hash_size - pad;
 
@@ -317,21 +328,25 @@ int _gnutls_ciphertext2TLSCompressed(GNUTLS_STATE state,
 			gnutls_assert();
 			return GNUTLS_E_RECEIVED_BAD_MESSAGE;
 		}
-		data = gnutls_malloc(length);
-		if (data==NULL) {
-			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
-		}
-
-		memcpy(data, ciphertext.data, length);
-		compress->data = data;
-		compress->size = length;
 		break;
 	default:
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 	}
 
+
+	data = gnutls_malloc(length);
+	if (data==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+
+	memcpy(data, ciphertext.data, length);
+	compress->data = data;
+	compress->size = length;
+
+
+
 	c_length = CONVuint16((uint16) compress->size);
 	seq_num = CONVuint64( &state->connection_state.read_sequence_number);
 
