@@ -73,7 +73,8 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_EXPIRED, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_HASH_FAILED, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_PARSING_ERROR, 1),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_AUTH_FAILED, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_PULL_ERROR, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_PUSH_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_NO_CERTIFICATE_FOUND, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_RECORD_LIMIT_REACHED, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_ASN1_PARSING_ERROR, 1),
@@ -82,10 +83,12 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_X509_UNSUPPORTED_CRITICAL_EXTENSION, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_X509_KEY_USAGE_VIOLATION, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_AGAIN, 0),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_INTERRUPTED, 0),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_REHANDSHAKE, 0),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_GOT_APPLICATION_DATA, 0),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_DB_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_PARAMETERS, 1),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_REQUEST, 1),
 	{0}
 };
 
@@ -100,7 +103,7 @@ static gnutls_error_entry error_algorithms[] = {
 
 /**
   * gnutls_is_fatal_error - Returns non-zero in case of a fatal error
-  * @error: is an error returned by a gnutls function. Error is always a negative value.
+  * @error: is an error returned by a gnutls function. Error should be a negative value.
   *
   * If a function returns a negative value you may feed that value
   * to this function to see if it is fatal. Returns 1 for a fatal 
