@@ -36,7 +36,6 @@
 #define GNUTLS_E_HASH_FAILED -33
 #define GNUTLS_E_PARSING_ERROR -34
 #define	GNUTLS_E_MPI_PRINT_FAILED -35
-#define GNUTLS_E_AUTH_FAILED -36
 #define GNUTLS_E_REHANDSHAKE -37
 #define GNUTLS_E_GOT_APPLICATION_DATA -38
 #define GNUTLS_E_RECORD_LIMIT_REACHED -39
@@ -52,5 +51,9 @@
 #define GNUTLS_E_PKCS1_WRONG_PAD -48
 #define GNUTLS_E_NO_CERTIFICATE_FOUND -49
 #define GNUTLS_E_INVALID_PARAMETERS -50
+#define GNUTLS_E_INVALID_REQUEST -51
+#define GNUTLS_E_INTERRUPTED -52
+#define GNUTLS_E_PUSH_ERROR -53
+#define GNUTLS_E_PULL_ERROR -54
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
