@@ -39,6 +39,7 @@
 #include "gnutls_cert.h"
 #include "gnutls_constate.h"
 #include <ext_dnsname.h>
+#include <gnutls_record.h>
 
 #ifdef HANDSHAKE_DEBUG
 #define ERR(x, y) _gnutls_log( "GNUTLS Error: %s (%d)\n", x,y)
@@ -133,14 +134,14 @@ void *_gnutls_ssl3_finished(GNUTLS_STATE state, int type, int skip)
 					   state->security_parameters.
 					   master_secret, 48);
 
-	siz = gnutls_getHashDataBufferSize(state) - skip;
+	siz = gnutls_get_handshake_buffer_size(state) - skip;
 	data = gnutls_malloc(siz);
 	if (data==NULL) {
 		gnutls_assert();
 		return NULL;
 	}
 
-	gnutls_readHashDataFromBuffer(state, data, siz);
+	gnutls_read_handshake_buffer(state, data, siz);
 
 	gnutls_mac_ssl3(td, data, siz);
 	gnutls_mac_ssl3(td2, data, siz);
@@ -180,14 +181,14 @@ void *_gnutls_finished(GNUTLS_STATE state, int type, int skip)
 	td = gnutls_hash_init(GNUTLS_MAC_MD5);
 	td2 = gnutls_hash_init(GNUTLS_MAC_SHA);
 
-	siz = gnutls_getHashDataBufferSize(state) - skip;
+	siz = gnutls_get_handshake_buffer_size(state) - skip;
 	data = gnutls_malloc(siz);
 	if (data==NULL) {
 		gnutls_assert();
 		return NULL;
 	}
 
-	gnutls_readHashDataFromBuffer(state, data, siz);
+	gnutls_read_handshake_buffer(state, data, siz);
 
 	gnutls_hash(td, data, siz);
 	gnutls_hash(td2, data, siz);
@@ -569,8 +570,8 @@ int _gnutls_send_empty_handshake(SOCKET cd, GNUTLS_STATE state, HandshakeType ty
 opaque data=0;
 opaque * ptr;
 
-if (again==0) ptr = &data;
-else ptr = NULL;
+	if (again==0) ptr = &data;
+	else ptr = NULL;
 
 	return _gnutls_send_handshake( cd, state, ptr, 0, type);
 }
@@ -587,9 +588,7 @@ int _gnutls_send_handshake(SOCKET cd, GNUTLS_STATE state, void *i_data,
 		/* we are resuming a previously interrupted
 		 * send.
 		 */
-		return
-		    _gnutls_handshake_send_int(cd, state, GNUTLS_HANDSHAKE, type,
-				       i_data, i_datasize);
+		return _gnutls_flush( cd, state);
 	}
 
 	if (i_data==NULL && i_datasize > 0) {
@@ -622,7 +621,7 @@ int _gnutls_send_handshake(SOCKET cd, GNUTLS_STATE state, void *i_data,
 	if (type != GNUTLS_HELLO_REQUEST) {
 
 		if ((ret =
-		     gnutls_insertHashDataBuffer(state, data,
+		     gnutls_insert_to_handshake_buffer(state, data,
 						 datasize)) < 0) {
 			gnutls_assert();
 			gnutls_free(data);
@@ -655,8 +654,8 @@ static int _gnutls_recv_handshake_header(SOCKET cd, GNUTLS_STATE state,
 
 	/* if we have data into the buffer then return them, do not read the next packet
 	 */
-	if (state->gnutls_internals.handshake_header_buffer.header_size >
-	    0) {
+	if (state->gnutls_internals.handshake_header_buffer.header_size ==
+	    handshake_header_size) {
 		*recv_type =
 		    state->gnutls_internals.handshake_header_buffer.
 		    recv_type;
@@ -666,36 +665,43 @@ static int _gnutls_recv_handshake_header(SOCKET cd, GNUTLS_STATE state,
 		    packet_length;
 	}
 
+	/* Note: SSL2_HEADERS == 1 */
+
 	dataptr = state->gnutls_internals.handshake_header_buffer.header;
 
-	ret =
-	    _gnutls_handshake_recv_int(cd, state, GNUTLS_HANDSHAKE, type,
+	/* If we haven't already read the handshake headers.
+	 */
+	if (state->gnutls_internals.handshake_header_buffer.header_size < SSL2_HEADERS) {
+		ret =
+		    _gnutls_handshake_recv_int(cd, state, GNUTLS_HANDSHAKE, type,
 				       dataptr, SSL2_HEADERS);
-	if (ret <= 0) {
-		gnutls_assert();
-		return (ret < 0) ? ret : GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+	
+		if (ret <= 0) {
+			gnutls_assert();
+			return (ret < 0) ? ret : GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+		}
+
+		if (ret != SSL2_HEADERS) {
+			gnutls_assert();
+			return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+		}
+		state->gnutls_internals.handshake_header_buffer.header_size += SSL2_HEADERS;
 	}
 
-	if (ret != SSL2_HEADERS) {
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
-
-	if (state->gnutls_internals.v2_hello == 0
-	    || type != GNUTLS_CLIENT_HELLO) {
+	if (state->gnutls_internals.v2_hello == 0 || type != GNUTLS_CLIENT_HELLO) {
 		ret =
 		    _gnutls_handshake_recv_int(cd, state, GNUTLS_HANDSHAKE,
 					       type,
-					       &dataptr[SSL2_HEADERS],
+					       &dataptr[state->gnutls_internals.handshake_header_buffer.header_size],
 					       HANDSHAKE_HEADER_SIZE -
-					       SSL2_HEADERS);
+					       state->gnutls_internals.handshake_header_buffer.header_size);
 		if (ret <= 0) {
 			gnutls_assert();
 			return (ret <
 				0) ? ret :
 			    GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 		}
-		if (ret != HANDSHAKE_HEADER_SIZE - SSL2_HEADERS) {
+		if (ret != HANDSHAKE_HEADER_SIZE - state->gnutls_internals.handshake_header_buffer.header_size) {
 			gnutls_assert();
 			return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 		}
@@ -729,7 +735,7 @@ static int _gnutls_recv_handshake_header(SOCKET cd, GNUTLS_STATE state,
 
 	if (*recv_type != GNUTLS_HELLO_REQUEST) {
 		if ((ret =
-		     gnutls_insertHashDataBuffer(state, dataptr,
+		     gnutls_insert_to_handshake_buffer(state, dataptr,
 						 handshake_header_size)) <
 		    0) {
 			gnutls_assert();
@@ -822,7 +828,7 @@ int _gnutls_recv_handshake(SOCKET cd, GNUTLS_STATE state, uint8 ** data,
 
 	if (recv_type != GNUTLS_HELLO_REQUEST && length32 > 0) {
 		if ((ret =
-		     gnutls_insertHashDataBuffer(state, dataptr,
+		     gnutls_insert_to_handshake_buffer(state, dataptr,
 						 length32)) < 0) {
 			gnutls_assert();
 			return ret;
@@ -1155,8 +1161,6 @@ static int _gnutls_send_client_hello(SOCKET cd, GNUTLS_STATE state, int again)
 				   GNUTLS_CLIENT_HELLO);
 	gnutls_free(data);
 
-
-
 	return ret;
 }
 
@@ -1319,13 +1323,6 @@ int _gnutls_recv_hello(SOCKET cd, GNUTLS_STATE state, char *data,
  *
  */
 
-
-#define STATE state->gnutls_internals.handshake_state
-/* This returns true if we have got there
- * before (and not finished due to an interrupt).
- */
-#define AGAIN(target) STATE==target?1:0
-
 /**
   * gnutls_rehandshake - This function will renegotiate security parameters
   * @cd: is a connection descriptor, as returned by socket().
@@ -1376,6 +1373,10 @@ int gnutls_rehandshake(SOCKET cd, GNUTLS_STATE state)
   * is checked automatically.
   * This function will fail if any problem is encountered,
   * and the connection should be terminated.
+  *
+  * This function may also return the non-fatal errors GNUTLS_E_AGAIN, or 
+  * GNUTLS_E_INTERRUPTED, and you may resume this handshake later.
+  *
   **/
 int gnutls_handshake(SOCKET cd, GNUTLS_STATE state)
 {
@@ -1401,10 +1402,10 @@ int gnutls_handshake(SOCKET cd, GNUTLS_STATE state)
 
 #define IMED_RET( str, ret) \
 	if (ret < 0) { \
+		if (gnutls_is_fatal_error(ret)==0) return ret; \
 		gnutls_assert(); \
 		ERR( str, ret); \
-		if (gnutls_is_fatal_error(ret)==0) return ret; \
-		gnutls_clearHashDataBuffer(state); \
+		gnutls_clear_handshake_buffer(state); \
 		return ret; \
 	}
 
@@ -1431,11 +1432,10 @@ int gnutls_handshake_client(SOCKET cd, GNUTLS_STATE state)
 					    session_id_size));
 #endif
 
-	if (STATE==STATE0) STATE = STATE1;
-
 	switch( STATE) {
+	case STATE0:
 	case STATE1:
-		ret = _gnutls_send_hello(cd, state, 0);
+		ret = _gnutls_send_hello(cd, state, AGAIN(STATE1));
 		STATE = STATE1;
 		IMED_RET("send hello", ret);
 
@@ -1520,7 +1520,6 @@ int gnutls_handshake_client(SOCKET cd, GNUTLS_STATE state)
 		IMED_RET("send client certificate verify", ret);
 
 		STATE = STATE0;
-	default:
 	}
 
 		
@@ -1535,11 +1534,11 @@ static int _gnutls_send_handshake_final(SOCKET cd, GNUTLS_STATE state,
 	int ret = 0;
 
 	/* Send the CHANGE CIPHER SPEC PACKET */
-	if (STATE==STATE0) STATE = STATE20;
 
 	switch( STATE) {
+	case STATE0:
 	case STATE20:
-		ret = _gnutls_send_change_cipher_spec(cd, state);
+		ret = _gnutls_send_change_cipher_spec(cd, state, AGAIN(STATE20));
 		STATE = STATE20;
 		if (ret < 0) {
 			ERR("send ChangeCipherSpec", ret);
@@ -1574,7 +1573,6 @@ static int _gnutls_send_handshake_final(SOCKET cd, GNUTLS_STATE state,
 		}
 
 		STATE = STATE0;
-	default:
 	}
 
 	return 0;
@@ -1588,11 +1586,10 @@ static int _gnutls_recv_handshake_final(SOCKET cd, GNUTLS_STATE state,
 					int init)
 {
 	int ret = 0;
-	char ch;
-
-	if (STATE==STATE0) STATE = STATE30;
+	uint8 ch;
 
 	switch(STATE) {
+	case STATE0:
 	case STATE30:
 		ret =
 		    gnutls_recv_int(cd, state, GNUTLS_CHANGE_CIPHER_SPEC, -1,
@@ -1628,7 +1625,6 @@ static int _gnutls_recv_handshake_final(SOCKET cd, GNUTLS_STATE state,
 			return ret;
 		}
 		STATE = STATE0;
-	default:
 	}
 
 	
@@ -1644,9 +1640,8 @@ int gnutls_handshake_server(SOCKET cd, GNUTLS_STATE state)
 {
 	int ret = 0;
 
-	if (STATE==STATE0) STATE = STATE1;
-
 	switch( STATE) {
+	case STATE0:
 	case STATE1:
 		ret =
 		    _gnutls_recv_handshake(cd, state, NULL, NULL,
@@ -1732,7 +1727,6 @@ int gnutls_handshake_server(SOCKET cd, GNUTLS_STATE state)
 		IMED_RET("recv client certificate verify", ret);
 
 		STATE = STATE0; /* finished thus clear state */
-	default:
 	}
 	
 	return 0;
@@ -1769,7 +1763,7 @@ int gnutls_handshake_common(SOCKET cd, GNUTLS_STATE state)
 		_gnutls_server_register_current_session(state);
 	}
 	/* clear handshake buffer */
-	gnutls_clearHashDataBuffer(state);
+	gnutls_clear_handshake_buffer(state);
 	return ret;
 
 }
@@ -1836,9 +1830,8 @@ int _gnutls_recv_hello_request(SOCKET cd, GNUTLS_STATE state, void *data,
 
 /* This function will remove algorithms that are not supported by
  * the requested authentication method. We only remove algorithm if
- * we receive client hello extensions (dnsname).
- *
- * or if we have a certificate with keyUsage bits (not ready yet)
+ * we receive client hello extensions (dnsname),
+ * or if we have a certificate with keyUsage bits set.
  */
 int _gnutls_remove_unwanted_ciphersuites(GNUTLS_STATE state,
 					 GNUTLS_CipherSuite **
