@@ -25,12 +25,13 @@
 #include <defines.h>
 
 /*
-#define IO_DEBUG 
+#define IO_DEBUG 5 // define this to check non blocking behaviour 
 #define BUFFERS_DEBUG
 #define HARD_DEBUG
 #define READ_DEBUG
 #define WRITE_DEBUG
-#define HANDSHAKE_DEBUG
+#define READ_DEBUG
+#define HANDSHAKE_DEBUG // Prints some information on handshake 
 #define RECORD_DEBUG
 #define DEBUG
 */
@@ -40,7 +41,7 @@
  */
 #define SOCKET int
 
-#define LIST ...
+#define GNUTLS_LIST ...
 
 #define MAX32 4294967295
 #define MAX24 16777215
@@ -106,7 +107,7 @@ typedef enum CloseRequest { GNUTLS_SHUT_RDWR=0, GNUTLS_SHUT_WR=1 } CloseRequest;
 
 typedef enum HandshakeState { STATE0=0, STATE1, STATE2, STATE3, STATE4, STATE5,
 	STATE6, STATE7, STATE8, STATE9, STATE10, STATE11, STATE20=20, STATE21,
-	STATE30=30, STATE31, STATE50 } HandshakeState;
+	STATE30=30, STATE31, STATE50=50, STATE60=60, STATE61 } HandshakeState;
 
 typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_SERVER_HELLO,
 		     GNUTLS_CERTIFICATE=11, GNUTLS_SERVER_KEY_EXCHANGE,
@@ -142,9 +143,19 @@ typedef enum CompressionMethod { GNUTLS_NULL_COMPRESSION=1, GNUTLS_ZLIB } Compre
 typedef enum ValidSession { VALID_TRUE, VALID_FALSE } ValidSession;
 typedef enum ResumableSession { RESUME_TRUE, RESUME_FALSE } ResumableSession;
 
-
 /* STATE (stop) */
 
+
+/* Pull & Push functions defines: 
+ */
+typedef ssize_t (*PULL_FUNC)(SOCKET, void*, size_t);
+typedef ssize_t (*PUSH_FUNC)(SOCKET, const void*, size_t);
+/* Store & Retrieve functions defines: 
+ */
+typedef int (*DB_STORE_FUNC)(void*, gnutls_datum key, gnutls_datum data);
+typedef int (*DB_REMOVE_FUNC)(void*, gnutls_datum key);
+typedef gnutls_datum (*DB_RETR_FUNC)(void*, gnutls_datum key);
+
 typedef struct {
 	KXAlgorithm algorithm;
 	void* credentials;
@@ -369,6 +380,10 @@ typedef struct {
 	int				send_buffer_prev_size; /* holds the
 	                                * data written in the previous runs.
 	                                */
+	int				send_buffer_user_size; /* holds the
+	                                * size of the user specified data to
+	                                * send.
+	                                */
 
 					/* 0 if no peeked data was kept, 1 otherwise.
 					 */
@@ -411,6 +426,17 @@ typedef struct {
 	int				(*x509_client_cert_callback)(void*,void*,int, void*, int);
 	gnutls_cert			peer_cert;
 	int				max_handshake_data_buffer_size;
+	/* PUSH & PULL functions.
+	 */
+	PULL_FUNC _gnutls_pull_func;
+	PUSH_FUNC _gnutls_push_func;
+	/* STORE & RETRIEVE functions. Only used if other
+	 * backend than gdbm is used.
+	 */
+	DB_STORE_FUNC db_store_func;
+	DB_RETR_FUNC db_retrieve_func;
+	DB_REMOVE_FUNC db_remove_func;
+	void* db_ptr;
 } GNUTLS_INTERNALS;
 
 struct GNUTLS_STATE_INT {
@@ -437,9 +463,6 @@ svoid *gnutls_PRF( opaque * secret, int secret_size, uint8 * label,
 		  int total_bytes);
 void _gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version);
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state);
-ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, HandshakeType htype, const void* data, size_t sizeofdata);
-ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, HandshakeType, char* data, size_t sizeofdata);
-ssize_t _gnutls_send_change_cipher_spec(SOCKET cd, GNUTLS_STATE state);
 
 /* These macros return the advertized TLS version of
  * the peer.
