@@ -29,6 +29,7 @@
 #include <gnutls_datum.h>
 #include "debug.h"
 
+
 /* Do PKCS-1 RSA encryption. 
  * pkey is the public key and n the modulus.
  */
@@ -63,11 +64,30 @@ int _gnutls_pkcs1_rsa_encrypt(gnutls_datum * ciphertext, gnutls_datum plaintext,
 	psize = k - 3 - plaintext.size;
 
 	ps = &edata[2];
-	_gnutls_get_random(ps, psize, GNUTLS_WEAK_RANDOM);
-	for (i = 0; i < psize; i++) {
-		if (ps[i] == 0)
+	switch(btype) {
+	    case 2:
+		_gnutls_get_random(ps, psize, GNUTLS_WEAK_RANDOM);
+		for (i = 0; i < psize; i++) {
+			if (ps[i] == 0)
+				ps[i] = 0xff;
+		}
+		break;
+	    case 1:
+		for (i = 0; i < psize; i++)
 			ps[i] = 0xff;
+	        break;
+#ifdef ALLOW_BLOCK_0
+	    case 0:
+		for (i = 0; i < psize; i++) {
+			ps[i] = 0x00;
+		}
+	        break;
+#endif
+	    default:
+	        gnutls_assert();
+	        return GNUTLS_E_UNKNOWN_ERROR;
 	}
+	
 	ps[psize] = 0;
 	memcpy(&ps[psize + 1], plaintext.data, plaintext.size);
 
@@ -107,9 +127,9 @@ int _gnutls_pkcs1_rsa_encrypt(gnutls_datum * ciphertext, gnutls_datum plaintext,
 
 /* Do PKCS-1 RSA decryption. 
  * pkey is the private key and n the modulus.
+ * Can decrypt block type 1 and type packets.
  */
-
-int _gnutls_pkcs1_rsa_decrypt(gnutls_datum * plaintext, gnutls_datum ciphertext,
+int _gnutls_pkcs1_rsa_decrypt(gnutls_sdatum * plaintext, gnutls_datum ciphertext,
 		      MPI pkey, MPI n, int btype)
 {
 	int k, esize, i, ret;
@@ -156,6 +176,7 @@ int _gnutls_pkcs1_rsa_decrypt(gnutls_datum * plaintext, gnutls_datum ciphertext,
 	 * (use block type 'btype')
 	 */
 
+
 	edata[0] = 0;
 	esize++;
 
@@ -166,11 +187,30 @@ int _gnutls_pkcs1_rsa_decrypt(gnutls_datum * plaintext, gnutls_datum ciphertext,
 	}
 
 	ret = GNUTLS_E_DECRYPTION_FAILED;
-	for (i=2;i<esize;i++) {
-		if (edata[i]==0) { 
-			ret = 0;
-			break;
+	switch(btype) {
+	    case 2:
+		for (i = 2; i < esize; i++) {
+			if (edata[i] == 0) {
+				ret = 0;
+				break;
+			}
 		}
+		break;
+	    case 1:
+		for (i = 2; i < esize; i++) {
+			if (edata[i] == 0 && i > 2) {
+				ret = 0;
+				break;
+			}
+			if (edata[i] != 0xff) {
+				ret = GNUTLS_E_PKCS1_WRONG_PAD;
+				break;
+			}
+		}
+	        break;
+	    default:
+	        gnutls_assert();
+	        return GNUTLS_E_UNKNOWN_ERROR;
 	}
 	i++;
 	
