@@ -35,7 +35,6 @@
 #include "gnutls_record.h"
 #include "gnutls_datum.h"
 
-
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state) {
 GNUTLS_Version ver;
 	ver = state->connection_state.version;
@@ -56,7 +55,7 @@ void _gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version) {
   * if you have changed the default low water value (default is 1).
   * Normally you will not need that function. 
   * This function is only usefull if using berkeley style sockets.
-  * Otherwise it does nothing.
+  * Otherwise it must be called and set lowat to zero.
   *
   **/
 int gnutls_set_lowat(GNUTLS_STATE state, int num) {
@@ -64,8 +63,6 @@ int gnutls_set_lowat(GNUTLS_STATE state, int num) {
 	return 0;
 }
 
-extern ssize_t (*_gnutls_pull_func)( SOCKET, void*, size_t);
-
 /**
   * gnutls_init - This function initializes the state to null (null encryption etc...).
   * @con_end: is used to indicate if this state is to be used for server or 
@@ -109,10 +106,7 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 
 	(*state)->gnutls_internals.expire_time = DEFAULT_EXPIRE_TIME; /* one hour default */
 
-	if (_gnutls_pull_func==NULL)
-		gnutls_set_lowat((*state), DEFAULT_LOWAT); /* the default for tcp */
-	else
-		gnutls_set_lowat((*state), 0);
+	gnutls_set_lowat((*state), DEFAULT_LOWAT); /* the default for tcp */
 
 	gnutls_set_max_handshake_data_buffer_size( (*state), MAX_HANDSHAKE_DATA_BUFFER_SIZE);
 
@@ -413,34 +407,60 @@ int ret;
   * further sends will be disallowed. In order to reuse the TCP connection
   * you should wait for an EOF from the peer.
   *
+  * This function may also return GNUTLS_E_AGAIN, or GNUTLS_E_INTERRUPTED.
+  *
   **/
 int gnutls_bye(SOCKET cd, GNUTLS_STATE state, CloseRequest how)
 {
 	int ret = 0, ret2 = 0;
 
-	ret = gnutls_send_alert(cd, state, GNUTLS_WARNING, GNUTLS_CLOSE_NOTIFY);
+	switch (STATE) {
+		case STATE0:
+		case STATE60:
+			if (STATE==STATE60) {
+				ret = _gnutls_flush( cd, state);
+			} else {
+				ret = gnutls_send_alert(cd, state, GNUTLS_WARNING, GNUTLS_CLOSE_NOTIFY);
+				STATE = STATE60;
+			}
+
+			if (ret < 0)
+				return ret;
+		case STATE61:
+			if ( how == GNUTLS_SHUT_RDWR && ret >= 0) {
+				ret2 = gnutls_recv_int(cd, state, GNUTLS_ALERT, -1, NULL, 0); 
+				if (ret2 >= 0) state->gnutls_internals.may_read = 1;
+			}
+			STATE = STATE61;
+
+			if (ret2 < 0)
+				return ret2;
 
-	if ( how == GNUTLS_SHUT_RDWR && ret == 0) {
-		ret2 = gnutls_recv_int(cd, state, GNUTLS_ALERT, -1, NULL, 0); 
-		state->gnutls_internals.may_read = 1;
 	}
-	state->gnutls_internals.may_write = 1;
+
+	STATE = STATE0;
 	
-	return GMIN(ret, ret2);
+	state->gnutls_internals.may_write = 1;
+	return 0;
 }
 
 /* This function behave exactly like write(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
  * send (if called by the user the Content is specific)
  * It is intended to transfer data, under the current state.    
+ *
+ * Oct 30 2001: Removed capability to send data more than MAX_ENC_SIZE.
+ * This makes the function much easier to read, and more error resistant
+ * (there were cases were the old function could mess everything up).
+ * --nmav
+ *
  */
 ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, HandshakeType htype, const void *_data, size_t sizeofdata)
 {
 	uint8 *cipher;
-	const uint8 *ptr;
 	int cipher_size;
-	int ret = 0, retval = 0;
-	int Size, data2send;
+	int retval, ret;
+	int data2send;
 	uint8 headers[5];
 	const uint8 *data=_data;
 	GNUTLS_Version lver;
@@ -449,7 +469,7 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		gnutls_assert();
 		return GNUTLS_E_INVALID_PARAMETERS;
 	}
-	
+
 	if (state->gnutls_internals.valid_connection == VALID_FALSE || state->gnutls_internals.may_write != 0) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
@@ -478,77 +498,65 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, sizeofdata);
 #endif
 
-	data2send = sizeofdata;
-	ptr = data;
-	retval = 0;
+	if ( sizeofdata > MAX_ENC_LEN)
+		data2send = MAX_ENC_LEN;
+	else 
+		data2send = sizeofdata;
 
-	while( data2send > 0) {
-	
-		if (data2send - MAX_ENC_LEN >= 0) {
-			data2send -= MAX_ENC_LEN;
-			Size = MAX_ENC_LEN;
-		} else {
-			Size = data2send;
-			data2send = 0;
-		}
-
-		/* Only encrypt if we don't have data to send 
-		 * from the previous run. - probably interrupted.
-		 */
-		if (state->gnutls_internals.send_buffer.size == 0) {
-			cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, ptr, Size, &cipher, type);
-			if (cipher_size <= 0) {
-				gnutls_assert();
-				if (cipher_size==0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
-				return cipher_size; /* error */
-			}
-		} else {
-			/* order write buffered to write
-			 * the buffered data.
-			 */
-			cipher = NULL;
-			cipher_size = state->gnutls_internals.send_buffer.size +
-				state->gnutls_internals.send_buffer_prev_size;
-		}
-
-		ptr += Size;
-
-		if ( (ret=_gnutls_write_buffered(cd, state, cipher, cipher_size)) != cipher_size) {
-			gnutls_free( cipher);
-			if ( ret<0 && gnutls_is_fatal_error(ret)==0) {
-				/* If we have sent any data then return
-				 * that value.
-				 */
-				gnutls_assert();
-				if (retval > 0) {
-					gnutls_assert();
-					return retval;
-				}
-				return ret;
-			}
-			state->gnutls_internals.valid_connection = VALID_FALSE;
-			state->gnutls_internals.resumable = RESUME_FALSE;
+	/* Only encrypt if we don't have data to send 
+	 * from the previous run. - probably interrupted.
+	 */
+	if (state->gnutls_internals.send_buffer.size != 0) {
+		ret = _gnutls_flush(cd, state);
+	} else {
+		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, data, data2send, &cipher, type);
+		if (cipher_size <= 0) {
 			gnutls_assert();
-			return ret;
+			if (cipher_size==0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
+			return cipher_size; /* error */
 		}
 
-		retval += Size;
-		gnutls_free(cipher);
-
-#ifdef RECORD_DEBUG
-		_gnutls_log( "Record: Sent Packet[%d] %s(%d) with length: %d\n",
-		(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, cipher_size);
-#endif
+		retval = data2send;
+		state->gnutls_internals.send_buffer_user_size =	data2send;
 
 		/* increase sequence number
 		 */
-		if (uint64pp( &state->connection_state.write_sequence_number) !=0) {
+		if (uint64pp( &state->connection_state.write_sequence_number) != 0) {
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			gnutls_assert();
+			/* FIXME: Somebody has to do rehandshake before that.
+			 */
 			return GNUTLS_E_RECORD_LIMIT_REACHED;
 		}
 
+
+		ret = _gnutls_write_buffered(cd, state, cipher, cipher_size);
 	}
+		
+	if ( ret != cipher_size) {
+		gnutls_free( cipher);
+		if ( ret < 0 && gnutls_is_fatal_error(ret)==0) {
+			/* If we have sent any data then return
+			 * that value.
+			 */
+			gnutls_assert();
+			return ret;
+		}
+		state->gnutls_internals.valid_connection = VALID_FALSE;
+		state->gnutls_internals.resumable = RESUME_FALSE;
+		gnutls_assert();
+		return ret;
+	}
+
+	state->gnutls_internals.send_buffer_user_size = 0;
+
+	gnutls_free(cipher);
+
+#ifdef RECORD_DEBUG
+	_gnutls_log( "Record: Sent Packet[%d] %s(%d) with length: %d\n",
+	(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, cipher_size);
+#endif
+
 
 	return retval;
 }
@@ -556,16 +564,18 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 /* This function is to be called if the handshake was successfully 
  * completed. This sends a Change Cipher Spec packet to the peer.
  */
-ssize_t _gnutls_send_change_cipher_spec(SOCKET cd, GNUTLS_STATE state)
+ssize_t _gnutls_send_change_cipher_spec(SOCKET cd, GNUTLS_STATE state, int again)
 {
 	opaque data[1] = { GNUTLS_TYPE_CHANGE_CIPHER_SPEC };
 
 #ifdef HANDSHAKE_DEBUG
 	_gnutls_log( "Record: Sent ChangeCipherSpec\n");
 #endif
-
-	return gnutls_send_int( cd, state, GNUTLS_CHANGE_CIPHER_SPEC, -1, data, 1);
-
+	if (again==0)
+		return gnutls_send_int( cd, state, GNUTLS_CHANGE_CIPHER_SPEC, -1, data, 1);
+	else {
+		return _gnutls_flush( cd, state);
+	}
 }
 
 static int _gnutls_check_recv_type( ContentType recv_type) {
@@ -603,7 +613,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	int header_size;
 
 	begin:
-	
+
 	header_size = RECORD_HEADER_SIZE;
 	ret = 0;
 
@@ -620,11 +630,11 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	/* If we have enough data in the cache do not bother receiving
 	 * a new packet. (in order to flush the cache)
 	 */
-	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && gnutls_getDataBufferSize(type, state) > 0) {
-		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
+	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && gnutls_get_data_buffer_size(type, state) > 0) {
+		ret = gnutls_get_data_buffer(type, state, data, sizeofdata);
 
 		/* if the buffer just got empty */
-		if (gnutls_getDataBufferSize(type, state)==0) {
+		if (gnutls_get_data_buffer_size(type, state)==0) {
 			if ( (ret2=_gnutls_clear_peeked_data( cd, state)) < 0) {
 				gnutls_assert();
 				return ret2;
@@ -787,7 +797,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	}
 
 	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
-		gnutls_insertDataBuffer(type, state, (void *) tmpdata, tmplen);
+		gnutls_insert_to_data_buffer(type, state, (void *) tmpdata, tmplen);
 	} else {
 		switch (recv_type) {
 		case GNUTLS_ALERT:
@@ -803,9 +813,9 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 				/* If we have been expecting for an alert do 
 				 * not call close().
 				 */
-				if (type != GNUTLS_ALERT)
-					gnutls_bye( cd, state, GNUTLS_SHUT_WR);
-
+				if (type != GNUTLS_ALERT) 
+					do ret=gnutls_bye( cd, state, GNUTLS_SHUT_WR);
+					while(ret==GNUTLS_E_INTERRUPTED || ret==GNUTLS_E_AGAIN);
 				gnutls_free(tmpdata);
 
 				return 0; /* EOF */
@@ -838,7 +848,10 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			return GNUTLS_E_UNEXPECTED_PACKET;
 		case GNUTLS_APPLICATION_DATA:
 			/* even if data is unexpected put it into the buffer */
-			gnutls_insertDataBuffer(recv_type, state, (void *) tmpdata, tmplen);
+			if ( (ret=gnutls_insert_to_data_buffer(recv_type, state, (void *) tmpdata, tmplen)) < 0) {
+				gnutls_assert();
+				return ret;
+			}
 
 			gnutls_assert();
 			gnutls_free(tmpdata);
@@ -871,10 +884,10 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 
 	/* Get Application data from buffer */
 	if ((type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && (recv_type == type)) {
-		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
+		ret = gnutls_get_data_buffer(type, state, data, sizeofdata);
 
 		/* if the buffer just got empty */
-		if (gnutls_getDataBufferSize(type, state)==0) {
+		if (gnutls_get_data_buffer_size(type, state)==0) {
 			if ( (ret2 = _gnutls_clear_peeked_data( cd, state)) < 0) {
 				gnutls_assert();
 				return ret2;
@@ -1043,7 +1056,7 @@ AlertDescription gnutls_get_last_alert( GNUTLS_STATE state) {
   * If the EINTR is returned by the internal push function (write())
   * then GNUTLS_E_INTERRUPTED, will be returned. If GNUTLS_E_INTERRUPTED or
   * GNUTLS_E_AGAIN is returned you must call this function again, with the 
-  * same (exactly) parameters. Otherwise the write operation will be 
+  * same parameters. Otherwise the write operation will be 
   * corrupted and the connection will be terminated.
   *
   * Returns the number of bytes received, zero on EOF, or
