@@ -668,7 +668,7 @@ _asn1_ordering_set(unsigned char *der,node_asn *node)
 
   first=last=NULL;
   while(p){
-    p_vet=(struct vet *)malloc(sizeof(struct vet));
+    p_vet=(struct vet *)gnutls_malloc(sizeof(struct vet));
     p_vet->next=NULL;
     p_vet->prev=last;
     if(first==NULL) first=p_vet;
@@ -696,11 +696,11 @@ _asn1_ordering_set(unsigned char *der,node_asn *node)
     while(p2_vet){
       if(p_vet->value>p2_vet->value){
 	/* change position */
-	temp=(unsigned char *)malloc(p_vet->end-counter);
+	temp=(unsigned char *)gnutls_malloc(p_vet->end-counter);
 	memcpy(temp,der+counter,p_vet->end-counter);
 	memmove(der+counter,der+p_vet->end,p2_vet->end-p_vet->end);
 	memcpy(der+p_vet->end,temp,p_vet->end-counter);
-	free(temp);
+	gnutls_free(temp);
 	
 	tag=p_vet->value;
 	p_vet->value=p2_vet->value;
@@ -716,7 +716,7 @@ _asn1_ordering_set(unsigned char *der,node_asn *node)
 
     if(p_vet!=first) p_vet->prev->next=NULL;
     else first=NULL;
-    free(p_vet);
+    gnutls_free(p_vet);
     p_vet=first;
   }
 }
@@ -748,7 +748,7 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
 
   first=last=NULL;
   while(p){
-    p_vet=(struct vet *)malloc(sizeof(struct vet));
+    p_vet=(struct vet *)gnutls_malloc(sizeof(struct vet));
     p_vet->next=NULL;
     p_vet->prev=last;
     if(first==NULL) first=p_vet;
@@ -786,11 +786,11 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
 
       if(change==1){
 	/* change position */
-	temp=(unsigned char *)malloc(p_vet->end-counter);
+	temp=(unsigned char *)gnutls_malloc(p_vet->end-counter);
 	memcpy(temp,der+counter,p_vet->end-counter);
 	memmove(der+counter,der+p_vet->end,p2_vet->end-p_vet->end);
 	memcpy(der+p_vet->end,temp,p_vet->end-counter);
-	free(temp);
+	gnutls_free(temp);
 	
 	p_vet->end=counter+(p2_vet->end-p_vet->end);
       }
@@ -802,7 +802,7 @@ _asn1_ordering_set_of(unsigned char *der,node_asn *node)
 
     if(p_vet!=first) p_vet->prev->next=NULL;
     else first=NULL;
-    free(p_vet);
+    gnutls_free(p_vet);
     p_vet=first;
   }
 }
@@ -1131,10 +1131,10 @@ asn1_get_der(node_asn *root,unsigned char *der,int len)
 	tag=_asn1_get_tag_der(der+counter,&class,&len2);
 	len2+=_asn1_get_length_der(der+counter+len2,&len3);
 	_asn1_length_der(len2+len3,NULL,&len4);
-	temp2=(unsigned char *)malloc(len2+len3+len4);
+	temp2=(unsigned char *)gnutls_malloc(len2+len3+len4);
 	_asn1_octet_der(der+counter,len2+len3,temp2,&len4);
 	_asn1_set_value(p,temp2,len4);
-	free(temp2);
+	gnutls_free(temp2);
 	counter+=len2+len3;
 	move=RIGHT;
 	break;
